@@ -458,6 +458,59 @@ static void print_load_map(struct obj_file *f)
 	}
 }
 
+/* 
+ * This function checks for the case where a module is compiled with gcc 2.x.y and the kernel
+ * is compiled with gcc 3.z. This is a known problematic case due to all kinds of ABI dependencies
+ * in the kernel on the gcc version.
+ * Returns 0 in the ok case, negative in the problem case
+ */
+static int check_gcc_mismatch(struct obj_file *f, char *filename)
+{
+	int module_gcc2_compiled = 0;
+	int kernel_gcc3_compiled = 0;
+	int i;
+	char kernel_line[256];
+	FILE *file;
+	
+	for (i = 0; i < HASH_BUCKETS; ++i) {
+		struct obj_symbol *sym = f->symtab[i];
+		for (sym = f->symtab[i]; sym ; sym = sym->next) {
+		    if (strstr(sym->name, "gcc2_compiled."))
+			module_gcc2_compiled = 1;
+		}
+    	}
+
+	/* If the module is all gcc 3 then no need to continue checking */
+	if (!module_gcc2_compiled)
+		return 0; 
+	memset(kernel_line,0,256);	
+	file = fopen("/proc/version","r");
+	if (!file)
+		return 0;
+	if (fread(kernel_line,1,255,file)<=0) 
+		return 0;
+	fclose(file);
+	
+	if (strstr(kernel_line,"gcc version 3"))
+		kernel_gcc3_compiled = 1;
+
+	if (module_gcc2_compiled && kernel_gcc3_compiled) {
+		if (flag_force_load) {
+			lprintf("Warning: The module you are trying to load (%s) is compiled with a gcc\n"
+				"version 2 compiler, while the kernel you are running is compiled with\n"
+				"a gcc version 3 compiler. This is known to not work.",filename);
+			++warnings;
+			return 0;
+		} else {
+			error("The module you are trying to load (%s) is compiled with a gcc\n"
+				"version 2 compiler, while the kernel you are running is compiled with\n"
+				"a gcc version 3 compiler. This is known to not work.",filename);
+			return -1;
+		}
+	}
+	return 0;
+}
+
 /************************************************************************/
 /* begin compat */
 
@@ -559,7 +612,8 @@ static void add_ksymtab(struct obj_file *f, struct obj_symbol *sym)
 	if (!sec)
 		return;
 	sec->header.sh_flags |= SHF_ALLOC;
-
+	sec->header.sh_addralign = tgt_sizeof_void_p;	/* Empty section might
+							   be byte-aligned */
 	ofs = sec->header.sh_size;
 	obj_symbol_patch(f, sec->idx, ofs, sym);
 	obj_string_patch(f, sec->idx, ofs + tgt_sizeof_void_p, sym->name);
@@ -1785,6 +1839,9 @@ int INSMOD_MAIN(int argc, char **argv)
 	if ((f = obj_load(fp, ET_REL, filename)) == NULL)
 		goto out;
 
+	if (check_gcc_mismatch(f, filename))
+		goto out;
+		
 	/* Version correspondence?  */
 	k_version = get_kernel_version(k_strversion);
 	m_version = get_module_version(f, m_strversion);
